#include <stdio.h>
#include <math.h>
#include "raylib.h"

#define SCREEN_MIDDLE ((Vector2) {window_size.x/2, window_size.y/2})

typedef struct Player {
    float radius;
    Vector2 coord;
} Player;

typedef struct Map {
    const Texture texture;
    const Rectangle box;
} Map;

typedef enum KeyControl {
    KC_UP = 0,
    KC_DOWN,
    KC_LEFT,
    KC_RIGHT,
    KC_SHOOT,
    KC_FREECAM,
    KC_CROUCH,
    KC_COUNT,
} KeyControl;

void change_layout_azerty(int *keys)
{
    keys[KC_UP]    = KEY_S;
    keys[KC_DOWN]  = KEY_W;
    keys[KC_LEFT]  = KEY_A;
    keys[KC_RIGHT] = KEY_D;
}

void change_layout_qwerty(int *keys)
{
    keys[KC_UP]    = KEY_W;
    keys[KC_DOWN]  = KEY_S;
    keys[KC_LEFT]  = KEY_A;
    keys[KC_RIGHT] = KEY_D;
}

Camera2D shake(Camera2D cam, int force)
{
    cam.offset.x += GetRandomValue(-force/2, force/2);
    cam.offset.y += GetRandomValue(-force/2, force/2);
    return cam;
}

int main(void)
{
    Vector2 window_size = { 800.0f, 600.0f };

    Camera2D camera = {
        .target = {0},
        .offset = SCREEN_MIDDLE,
        .rotation = 0.0f,
        .zoom = 1.0f,
    };

    Player player = {
        .radius = 25.0f,
    };

#ifdef RELEASE
    SetTraceLogLevel(LOG_FATAL);
#endif

    InitWindow(window_size.x, window_size.y, "voleur");
    SetWindowState(FLAG_WINDOW_RESIZABLE);
    SetTargetFPS(60);

#ifdef RELEASE
    SetExitKey(KEY_NULL);
#endif

    Map map = {
        .texture = LoadTexture("data/map2.png"),
        .box = (Rectangle) {
            .x = -map.texture.width/2, .y = -map.texture.height/2,
            .width = map.texture.width, .height = map.texture.height,
        },
    };

    int keys[KC_COUNT] = {0};
    keys[KC_SHOOT]   = MOUSE_BUTTON_LEFT;
    keys[KC_CROUCH]  = KEY_LEFT_SHIFT;
    keys[KC_FREECAM] = KEY_F;
    change_layout_azerty(keys);

    while (!WindowShouldClose()) {
        float DT = GetFrameTime();
        if (IsWindowResized()) {
            window_size = (Vector2) {GetScreenWidth(), GetScreenHeight()};
            camera.offset = SCREEN_MIDDLE;
        }

        float movement_speed = 10.0f * DT * 50;
        if ((IsKeyDown(keys[KC_UP]) || IsKeyDown(keys[KC_DOWN])) && (IsKeyDown(keys[KC_LEFT]) || IsKeyDown(keys[KC_RIGHT])))
            movement_speed = sqrtf(movement_speed) * 2;
        if (IsKeyDown(keys[KC_CROUCH]))
            movement_speed /= 2.0f;

        if (IsKeyDown(keys[KC_LEFT]))
            camera.target.x -= movement_speed ;
        if (IsKeyDown(keys[KC_RIGHT]))
            camera.target.x += movement_speed ;
        if (IsKeyDown(keys[KC_DOWN]))
            camera.target.y -= movement_speed ;
        if (IsKeyDown(keys[KC_UP]))
            camera.target.y += movement_speed ;

        if (IsKeyReleased(keys[KC_FREECAM]))
            camera.target = player.coord;
        if (!IsKeyDown(keys[KC_FREECAM])) {
            player.coord = GetScreenToWorld2D(SCREEN_MIDDLE, camera);

            if (player.coord.x - player.radius < map.box.x)
                camera.target.x = map.box.x + player.radius;
            else if (player.coord.x + player.radius > map.box.x + map.box.width)
                camera.target.x = map.box.x + map.box.width - player.radius;

            if (player.coord.y - player.radius < map.box.y)
                camera.target.y = map.box.y + player.radius;
            else if (player.coord.y + player.radius > map.box.y + map.box.height)
                camera.target.y = map.box.y + map.box.height - player.radius;

            player.coord = GetScreenToWorld2D(SCREEN_MIDDLE, camera);
        }

        Camera2D effect_camera = camera;
        if (IsMouseButtonDown(keys[KC_SHOOT]))
            effect_camera = shake(camera, 12);

        BeginDrawing();
        {
            ClearBackground(LIME);

            BeginMode2D(effect_camera);
            {
                DrawTexture(map.texture, map.box.x, map.box.y, WHITE);
                DrawCircle(player.coord.x, player.coord.y, player.radius, RED);
            }
            EndMode2D();
        }
        EndDrawing();
    }

    CloseWindow();
    return 0;
}
